﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Example02_Transform
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        // コンストラクター
        public MainWindow()
        {
            InitializeComponent();
            InitializeWindow();
        }

        private const double TIME = 3;        // 各変換のアニメーションの時間（片道）
        private AnimationClock _clockRot;     // 回転のAnimationClock
        private AnimationClock _clockScX;     // 横方向サイズ拡大のAnimationClock
        private AnimationClock _clockScY;     // 縦方向サイズ拡大のAnimationClock
        private AnimationClock _clockTrX;     // 横方向移動のAnimationClock
        private AnimationClock _clockTrY;     // 縦方向移動のAnimationClock

        // [Avi]ボタンクリックイベント
        private void btnAvi_Click(object sender, RoutedEventArgs e)
        {
            var filepath = GetAviFilePath();
            if (0 < filepath.Length)
            {
                SaveAsAvi(filepath);
                MessageBox.Show("OK");
            }
        }

        // [開始]ボタンクリックイベント
        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            _clockRot.Controller.Begin();
            _clockScX.Controller.Begin();
            _clockScY.Controller.Begin();
            _clockTrX.Controller.Begin();
            _clockTrY.Controller.Begin();
        }

        // [停止]ボタンクリックイベント
        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            _clockRot.Controller.Stop();
            _clockScX.Controller.Stop();
            _clockScY.Controller.Stop();
            _clockTrX.Controller.Stop();
            _clockTrY.Controller.Stop();
        }


        // Aviファイル保存パス取得
        private string GetAviFilePath()
        {
            var dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.Filter = "aviファイル|*.avi;*.avi";
            var path = String.Empty;

            if (dlg.ShowDialog() == true)
            {
                path = dlg.FileName;
            }

            return path;
        }

        // Ellipseの配置とアニメーションの設定
        private void InitializeWindow()
        {
            const double RADIUS = 20;

            // Ellipseの生成と配置
            var circle = new Ellipse();
            circle.Width = RADIUS * 2;
            circle.Height = RADIUS * 2;
            Canvas.SetLeft(circle, 0);
            Canvas.SetTop(circle, 0);
            var gscollect = new GradientStopCollection();
            gscollect.Add(new GradientStop(Colors.Blue, 0.5));
            gscollect.Add(new GradientStop(Colors.Red, 0.5));
            circle.Fill = new LinearGradientBrush(gscollect);
            cvs1.Children.Add(circle);

            // Transform関連オブジェクト生成
            var tgroup = new TransformGroup();
            circle.RenderTransform = tgroup;
            
            // 回転のアニメーション生成
            var rotTran = new RotateTransform(0, RADIUS, RADIUS);
            tgroup.Children.Add(rotTran);
            var animRot = new DoubleAnimation(0, 360, new Duration(TimeSpan.FromSeconds(TIME)));
            animRot.RepeatBehavior = RepeatBehavior.Forever;
            _clockRot = animRot.CreateClock();
            rotTran.ApplyAnimationClock(RotateTransform.AngleProperty, _clockRot);
            _clockRot.Controller.Stop();

            // サイズ拡大のアニメーション生成
            var scTran = new ScaleTransform();
            tgroup.Children.Add(scTran);
            var animSc = new DoubleAnimation(1, 2, new Duration(TimeSpan.FromSeconds(TIME)));
            animSc.AutoReverse = true;
            animSc.RepeatBehavior = RepeatBehavior.Forever;
            _clockScX = animSc.CreateClock();
            scTran.ApplyAnimationClock(ScaleTransform.ScaleXProperty, _clockScX);
            _clockScX.Controller.Stop();
            _clockScY = animSc.CreateClock();
            scTran.ApplyAnimationClock(ScaleTransform.ScaleYProperty, _clockScY);
            _clockScY.Controller.Stop();

            // 移動(X方向)のアニメーション生成
            var trTran = new TranslateTransform();
            tgroup.Children.Add(trTran);
            var animTrX = new DoubleAnimation(0, 400, new Duration(TimeSpan.FromSeconds(TIME)));
            animTrX.AutoReverse = true;
            animTrX.RepeatBehavior = RepeatBehavior.Forever;
            _clockTrX = animTrX.CreateClock();
            trTran.ApplyAnimationClock(TranslateTransform.XProperty, _clockTrX);
            _clockTrX.Controller.Stop();

            // 移動(Y方向)のアニメーション生成
            var animTrY = new DoubleAnimation(0, 180, new Duration(TimeSpan.FromSeconds(TIME)));
            animTrY.AutoReverse = true;
            animTrY.RepeatBehavior = RepeatBehavior.Forever;
            _clockTrY = animTrY.CreateClock();
            trTran.ApplyAnimationClock(TranslateTransform.YProperty, _clockTrY);
            _clockTrY.Controller.Stop();

        }

        // Aviファイル作成
        private void SaveAsAvi(string filepath)
        {
            var fps = 48;                                // フレームレート
            var totalFrames = (int)(fps * TIME * 2);     // 全フレーム数
            var secs = Enumerable.Range(0, totalFrames).Select(t => (((double)t) / fps));
            var aviManager = new AviFile.AviManager(filepath, false);
            AviFile.VideoStream aviStream = null;
            var tempFile = "frame.png";

            foreach (var sec in secs)
            {
                _clockRot.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                           TimeSeekOrigin.BeginTime);
                _clockScX.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                           TimeSeekOrigin.BeginTime);
                _clockScY.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                           TimeSeekOrigin.BeginTime);
                _clockTrX.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                           TimeSeekOrigin.BeginTime);
                _clockTrY.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                           TimeSeekOrigin.BeginTime);
                cvs1.UpdateLayout();
                cvs1.SaveImage(tempFile);
                var bmp = new System.Drawing.Bitmap(tempFile);
                if (aviStream == null)
                    aviStream = aviManager.AddVideoStream(true, fps, bmp);
                else
                    aviStream.AddFrame(bmp);
                bmp.Dispose();
            }
            aviManager.Close();

            _clockRot.Controller.Stop();
            _clockScX.Controller.Stop();
            _clockScY.Controller.Stop();
            _clockTrX.Controller.Stop();
            _clockTrY.Controller.Stop();

            System.IO.File.Delete(tempFile);
        }

    }

    public static class CanvasExtension
    {
        /// <summary> [SaveImage]：Canvasの画像をファイルに保存するCanvasクラスの拡張メソッド 
        /// 出典：http://qiita.com/hugo-sb/items/894914f6bbe224a45d49
        /// </summary>
        /// <param name="canvas"></param>
        /// <param name="path">保存する画像ファイル名</param>
        /// <param name="encoder">BitmapEncoderが指定されなかった場合はPNG形式になる</param>
        public static void SaveImage(this Canvas canvas, string path, BitmapEncoder encoder = null)
        {
            // レイアウトを再計算させる
            var size = new Size(canvas.ActualWidth, canvas.ActualHeight);
            canvas.Measure(size);
            canvas.Arrange(new Rect(size));

            // VisualObjectをBitmapに変換する
            var renderBitmap = new RenderTargetBitmap((int)size.Width,       // 画像の幅
                                                      (int)size.Height,      // 画像の高さ
                                                      96.0d,                 // 横96.0DPI
                                                      96.0d,                 // 縦96.0DPI
                                                      PixelFormats.Pbgra32); // 32bit(RGBA各8bit)
            renderBitmap.Render(canvas);

            // 出力用の FileStream を作成する
            using (var stream = new System.IO.FileStream(path, System.IO.FileMode.Create))
            {
                // 変換したBitmapをエンコードしてFileStreamに保存する。
                // BitmapEncoder が指定されなかった場合は、PNG形式とする。
                encoder = encoder ?? new PngBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(renderBitmap));
                encoder.Save(stream);
            }
        }

    }

}
